from django import forms
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.db import models

from .utils import base_n, generate_id


def get_max_length(chars, default=36):
    real_max_length = default
    if chars:
        max_base = len(chars)
        real_max_length = len(base_n(2 ** 128, max_base, chars))

    max_length = getattr(settings, 'PUBLIC_ID_MAX_LENGTH', real_max_length)
    if max_length < real_max_length:
        raise ImproperlyConfigured('PUBLIC_ID_MAX_LENGTH must be great {}'.format(real_max_length))

    return max_length


class PublicIdFormField(forms.SlugField):
    pass


class PublicIdField(models.SlugField):
    @property
    def allowed_chars(self):
        return getattr(settings, 'PUBLIC_ID_CHARS', None)

    def __init__(self, auto=False, *args, **kwargs):
        kwargs['max_length'] = get_max_length(self.allowed_chars)

        self.auto = auto
        if auto:
            kwargs['editable'] = False
            kwargs['blank'] = True
            kwargs['unique'] = True

        super(PublicIdField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        kwargs['form_class'] = PublicIdFormField
        return super(PublicIdField, self).formfield(**kwargs)

    def pre_save(self, model_instance, add):
        if self.auto and not getattr(model_instance, self.attname):
            value = generate_id(self.allowed_chars, get_max_length(self.allowed_chars))
            setattr(model_instance, self.attname, value)
            return value
        else:
            return super(PublicIdField, self).pre_save(model_instance, add)



import string
import uuid
import random

DEFAULT_CHARS = string.digits + string.ascii_lowercase


def generate_id(chars=None, length=36):
    if chars:
        rng = random.SystemRandom()
        alphabet_length = len(chars)

        indexes = [rng.randrange(alphabet_length) for _ in range(length)]

        return ''.join(chars[i] for i in indexes)
    else:
        return str(uuid.uuid4())


def base_n(num, base, chars=DEFAULT_CHARS):
    """
    >>> base_n(42, 10)
    '42'
    >>> base_n(42, 2)
    '101010'
    """
    if num == 0:
        return '0'
    if not 2 <= base <= len(chars):
        raise ValueError('Base must be between 2-%d' % len(chars))
    left_digits = num // base
    if left_digits == 0:
        return chars[num % base]
    else:
        return base_n(left_digits, base, chars) + chars[num % base]



#!/usr/bin/env python
import os
import sys

from setuptools import setup, find_packages

version = __import__('public_id').__version__

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    os.system('python setup.py bdist_wheel upload')
    sys.exit()


setup(
    name='django-public-id',
    description='Long non-incremental IDs for public links. Essentially a compact form of UUID (22 bytes for 128-bit value)',
    version=version,
    author='Maxim Oransky',
    author_email='maxim.oransky@gmail.com',
    maintainer='Dmitrii Gerasimenko',
    maintainer_email='kiddima@gmail.com',
    url='https://github.com/shantilabs/django-public-id',
    packages=['public_id'],
    zip_safe=True,
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Framework :: Django",
        "Intended Audience :: Developers",
    ],
)
